#!/usr/bin/perl -w

################################################################################
# config_lids.pl                                                March 07, 2001 #
#                                                                  Version 1.6 #
################################################################################
# Nick DeClario <nick@guardiandigital.com>                                     #
# Ryan W. Maple <ryan@guardiandigital.com>                                     #
#                                                                              #
# This script should be edited and run whenever the administrator wishes to    #
# change LIDS file protections.                                                #
################################################################################
# CHANGELOG                                                                    #
# 2001.03.07 (1.0) [rwm]:  Initial Version.                                    #
# 2001.03.07 (1.1) [rwm]:  Added '--force' and '--help' options.               #
# 2001.03.08 (1.2) [rwm]:  Added '/etc/init.d/named' to READ files.            #
# 2001.03.13 (1.3) [rwm]:  Added the rest of /etc/init.d/$i to READ files.     #
#                          Updated sysklogd (was syslog-ng/klogd) initscript.  #
# 2001.03.19 (1.4) [rwm]:  Added a 'WRITE' section.                            #
# 2001.03.28 (1.5) [rwm]:  Updated to reflect new lidsadm formats.             #
# 2002.01.14 (1.6) [rwm]:  Make LIDS configuration work again with kernel      #
#                          update.                                             #
################################################################################

  ### Be mean.
  use strict;
  use Crypt::RIPEMD160;
  use Getopt::Long;

  ### Setup local varibles.
  my $LIDSADM     =  '/sbin/lidsadm';
  my $LIDSADM_GD  =  '/sbin/lidsadm_gd';
  my $VERBOSE     =  0;

  my ($force, $usage);
  my $ret         =  GetOptions ( 'force' => \$force, 'help' => \$usage );


    ### Print help if we ask for it.
    if ($usage) {
      print STDERR "Usage:  ./config_lids.pl { --force | --help }\n";
      exit 0;
    }

    ### Basic sanity checks.
    if (! -x $LIDSADM) {
      print STDERR "$LIDSADM does not exist or is not executble.\n"; exit -1;
    }

    if (! -x $LIDSADM_GD) {
      print STDERR "$LIDSADM_GD does not exist or is not executble.\n"; exit -1;
    }


    ### Do not check status if we have specified the "--force" option.
    if (!$force) {
      my $STATUS  =  `$LIDSADM -V | grep LIDS_GLOBAL | awk '{ print \$2; }'`;
      chomp $STATUS;

      if ($STATUS != 0) {
        print STDERR "This script cannot run because LIDS is enabled globally.\n";
        exit -2;
      }
    }


  ### These are files that we want to setup as "DENY" (hidden).
  my @DENY     =  (
                    '/etc/lids',
                    '/etc/webtool',
                    '/usr/userpass',
                    '/usr/webtool'
                  );

  ### These are files that we want to setup as "READ only".
  my @READ     =  (
                    '/bin',
                    '/boot',
                    '/etc/fstab',
                    '/etc/init.d',
                    '/etc/init.d/bootmisc',
                    '/etc/init.d/checkfs',
                    '/etc/init.d/checkroot',
                    '/etc/init.d/console-screen',
                    '/etc/init.d/crond',
                    '/etc/init.d/devpts',
                    '/etc/init.d/functions',
                    '/etc/init.d/halt',
                    '/etc/init.d/harddisks',
                    '/etc/init.d/hostname',
                    '/etc/init.d/httpd',
                    '/etc/init.d/hwclock',
                    '/etc/init.d/ipchains',
                    '/etc/init.d/keytable',
                    '/etc/init.d/killall',
                    '/etc/init.d/modutils',
                    '/etc/init.d/mountall',
                    '/etc/init.d/mysql',
                    '/etc/init.d/named',
                    '/etc/init.d/network',
                    '/etc/init.d/nviboot',
                    '/etc/init.d/pacct',
                    '/etc/init.d/postfix',
                    '/etc/init.d/procps',
                    '/etc/init.d/random',
                    '/etc/init.d/reboot',
                    '/etc/init.d/rmnologin',
                    '/etc/init.d/sendsigs',
                    '/etc/init.d/single',
                    '/etc/init.d/snortd',
                    '/etc/init.d/sshd',
                    '/etc/init.d/stunnel-imap',
                    '/etc/init.d/stunnel-pop3',
                    '/etc/init.d/swatch',
                    '/etc/init.d/sys-unconfig',
                    '/etc/init.d/sysklogd',
                    '/etc/init.d/sysstat',
                    '/etc/init.d/umountfs',
                    '/etc/init.d/userpass',
                    '/etc/init.d/webtool',
                    '/etc/init.d/xinetd',
                    '/etc/init.d/xntpd',
                    '/etc/init.d/zlidsinit',
                    '/etc/inittab',
                    '/etc/lids',
                    '/etc/lilo.conf',
                    '/etc/postfix/postfix-script',
                    '/etc/pwdb.conf',
                    '/etc/rc',
                    '/etc/rcS',
                    '/etc/securetty',
                    '/etc/sysctl.conf',
                    '/lib',
                    '/usr',
                    '/usr/bin',
                    '/usr/bin/perl',
                    '/usr/sbin',
                    '/usr/sbin/webtool',
                    '/usr/sbin/userpass',
                    '/usr/webtool/miniserv.pl',
                    '/usr/userpass/userpass.pl',
                    '/usr/local',
                    '/sbin',
                    '/var/lib/rpm'
                  );

  ### These are files that we want to setup as "WRITE"able.
  my @WRITE     =  (
                     '/lib/modules'
                   );

  ### These are source/object associations.
  my @OBJECTS  = (
                   { SOURCE      => '/usr/userpass/userpass.pl',
                     OBJECT      => '/etc/webtool',
                     FLAG        => '-i 10',
                     PROTECTION  => 'WRITE' },

                   { SOURCE      => '/usr/userpass/userpass.pl',
                     OBJECT      => '/usr/userpass',
                     FLAG        => '-i 10',
                     PROTECTION  => 'WRITE' },

                   { SOURCE      => '/usr/webtool/miniserv.pl',
                     OBJECT      => '/etc/init.d',
                     FLAG        => '-i 10',
                     PROTECTION  => 'WRITE' },

                   { SOURCE      => '/usr/webtool/miniserv.pl',
                     OBJECT      => '/etc/lids',
                     FLAG        => '-i 10',
                     PROTECTION  => 'WRITE' },

                   { SOURCE      => '/usr/webtool/miniserv.pl',
                     OBJECT      => '/etc/webtool',
                     FLAG        => '-i 10',
                     PROTECTION  => 'WRITE' },

                   { SOURCE      => '/usr/webtool/miniserv.pl',
                     OBJECT      => '/usr/webtool',
                     FLAG        => '-i 10',
                     PROTECTION  => 'WRITE' },
                 );


################################################################################
# NOTHING SHOULD BE EDITED BELOW THIS LINE #####################################
################################################################################

  ### Step 1: Clear out all rules.
  my $rc  =  system "$LIDSADM -Z > /dev/null 2>&1";
  print STDERR "Cleared all rules.\n\n" if ($rc == 0);


  ### Step 2:  Set up READ permissions.
  print STDERR "Setting up READ permissions...\n";
  foreach my $f (@READ) {
    my $rc  =  system "$LIDSADM -A -o $f -j READ > /dev/null 2>&1";
    if ($rc != 0) { &handle_error($f, $rc/256); }
  }


  ### Step 3:  Set up WRITE permissions.
  print STDERR "Setting up WRITE permissions...\n";
  foreach my $f (@WRITE) {
    my $rc  =  system "$LIDSADM -A -o $f -j WRITE > /dev/null 2>&1";
    if ($rc != 0) { &handle_error($f, $rc/256); }
  }


  ### Step 4:  Set up DENY permissions.
  print STDERR "Setting up DENY permissions...\n";
  foreach my $f (@DENY) {
    my $rc  =  system "$LIDSADM -A -o $f -j DENY > /dev/null 2>&1";
    if ($rc != 0) { &handle_error($f, $rc/256); }
  }


  ### Step 5:  Set up subject/object associations.
  print STDERR "Setting up subject/object associations...\n";
  foreach my $o (@OBJECTS) {
    my $S  =  $o->{'SOURCE'};
    my $O  =  $o->{'OBJECT'};
    my $F  =  $o->{'FLAG'};
    my $P  =  $o->{'PROTECTION'};

    my $rc  =  system "$LIDSADM -A -s $S -o $O $F -j $P > /dev/null 2>&1";
    print STDERR "ERROR:  $S:$O ($P)\n" if ($rc != 0);
    print STDERR "  $LIDSADM -A -s $S -o $O $F -j $P\n\n" if ($rc != 0);
  }


  ### Step 6:  DENY all SSL certificates and keys, and provide necessary access.
  print STDERR "Setting up certificate/key protection...\n";
  my $basedir   =  '/home/httpd';
  my $httpdbin  =  '/usr/sbin/httpd';
  my $wtbin     =  '/usr/webtool/miniserv.pl';

  chdir $basedir;
  my @dirs  =  glob "*";

  foreach my $d (@dirs) {
    if ( ( -e "$d/ssl/certificate.pem" ) && ( -e "$d/ssl/key.pem" ) ) {

      my $rc;
      my $ssldir  =  "$basedir/$d/ssl/";

        $rc  =  system "$LIDSADM -A -o $ssldir -j DENY > /dev/null 2>&1";
        print STDERR "ERROR4.1:  ($ssldir)\n" if ($rc != 0);
        $rc  =  system "$LIDSADM -A -s $httpdbin -o $ssldir -j READ >/dev/null 2>&1";
        print STDERR "ERROR4.2:  ($ssldir)\n" if ($rc != 0);
        $rc  =  system "$LIDSADM -A -s $wtbin -o $ssldir -i 20 -j WRITE > /dev/null 2>&1";
        print STDERR "ERROR4.3:  ($ssldir)\n" if ($rc != 0);
    }
  }


  ### Step 7:  Reload the LIDS configuration file (which now has changes).
  print STDERR "\nReloading LIDS configuration...\n";
  my $lidspass = &get_lids_passwd();
  system "$LIDSADM_GD -Q -- +RELOAD_CONF $lidspass > /dev/null 2>&1";


  ### Exit quietly.
  exit 0;

################################################################################
# handle_error(FILENAME, RETURN_CODE) ##########################################
################################################################################
# If an error is encountered setting up a READ permission, this subroutine 
# tells the user (the best it can) what sort of error was encountered.
################################################################################
sub handle_error() {
  my ($file, $rc) = @_;

  if (! -f $file) { print STDERR "$file: No such file or directory.\n"; }
  else            { print STDERR "$file: Unspecified error.\n"; }
}

################################################################################
# get_lids_pass() ##############################################################
################################################################################
# This subroutine returns the encrypted password for use in +RELOAD_CONF. 
################################################################################
sub get_lids_passwd() {
  my $lidspass  =  undef;

  if ( -f "/etc/webtool/SystemManager/config" ) {
    open FILE, "/etc/webtool/SystemManager/config";
      while (<FILE>) { if ($_ =~ m/^pass\=(.*)$/) { $lidspass = $1; } }
    close FILE;
  }

  if (!$lidspass) {
    print STDERR "You must reload your LIDS configuration manually.\n";
    exit -3;
  }

  my $first_hash   =  new Crypt::RIPEMD160;

  $first_hash->add($lidspass);
  my $digest     = $first_hash->digest();;
  my $digest_str = unpack "H*", $digest;

  return $digest_str;
}

